/*
  a generic producer for deviceConfig entries
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */




#ifndef __DEVICECONFIGPRODUCER_H
#define __DEVICECONFIGPRODUCER_H


#include <producer/core/threadState.h>
#include <deviceConfig/deviceConfig.h>


/**
  What is this: A producer?
  <p>
  A producer create asynchron new deviceConfigs. A server enters
  the producer to get a snapshot of the current asynchron state.
  <p>
  After the server exists the data in the server keeps the same 
  until the next entry.
  In the meantime the producer can -asynchronly- change its 
  deviceConfig.
  <p>
  
  The producer is the main work if you write a new interface.
  Most of the deviceConfig class is highly depend on what 
  your classe does.
  <p>
  Eg: A LineIn reader never creates musicInfo, because this
  lineIn stream simply has no info about the music.
  A lineIn producer cannot use a ringBuffer, because each
  time the thread Enters and want pcm data the data is 
  generated "on the fly"
  <p>
  A producer must coordinate between the  user commands an the current
  deviceConfig. It sometime must stop the thread because it
  cannot fill the deviceConfig and after a time it must wake
  it up.
  <p>
  The main reason why there a two classes (a server and a producer)
  is because the server must keep the valid deviceConfig class
  whereas the "state" of the deviceConfig in the producer
  is not clear.
  <p>
  
*/



class DeviceConfigProducer   {

  ThreadState* client;

 public:
  DeviceConfigProducer();
  virtual ~DeviceConfigProducer();

  virtual void setNotifyClient(ThreadState* client);
  ThreadState* getNotifyClient();
  
  virtual void updateDeviceConfig(DeviceConfig* newConfig);


 protected:
  virtual void setProducerEnter(int allow);

};


#endif













